#include "log.h"
#include "format.h"
#include "numUtil.h"


/** 
 *   \file numUtil.c Numeric utilities
 *   \author Lukas Habegger (lukas.habegger@yale.edu)
 */



/** 
 * Returns rounded a*p/q 
 */
int roundingScale(int a, int p, int q)
{
  if (a > 100000 || p > 100000)
    {
      double x = a;
      x *= p;
      x /= q;
      return round(x);
    }
  else
    return (a*p + q/2)/q;
}



/** 
 * Return amount of bases two ranges intersect over, 0 or negative if no intersection. 
 */
int  rangeIntersection(int start1, int end1, int start2, int end2)
{
  int s = MAX(start1,start2);
  int e = MIN(end1,end2);
  return e-s;
}



/** 
 * Return number of bases in intersection of two ranges, or zero if they don't intersect. 
 */
int positiveRangeIntersection(int start1, int end1, int start2, int end2)

{
  int ret = rangeIntersection(start1,end1,start2,end2);
  if (ret < 0)
    ret = 0;
  return ret;
}



/** 
 * Return byte-swapped version of a. 
 */
unsigned byteSwap32(unsigned a)
{
  union {unsigned whole; unsigned char bytes[4];} u,v;
  u.whole = a;
  v.bytes[0] = u.bytes[3];
  v.bytes[1] = u.bytes[2];
  v.bytes[2] = u.bytes[1];
  v.bytes[3] = u.bytes[0];
  return v.whole;
}



/**
 * Return base two number of digits. 
 */
int digitsBaseTwo(unsigned long x)
{
  int digits = 0;
  while (x)
    {
      digits += 1;
      x >>= 1;
    }
  return digits;
}



/**
 * Return number of digits base 10. 
 */
int digitsBaseTen(int x)
{
  int digCount = 1;
  if (x < 0)
    {
      digCount = 2;
      x = -x;
    }
  while (x >= 10)
    {
      digCount += 1;
      x /= 10;
    }
  return digCount;
}
